@@ -29,6 +29,7 @@
 #include <droute/droute.h>
 
 #include "common/spi-dbus.h"
+#include "common/spi-stateset.h"
 #include "accessible-cache.h"
 #include "bridge.h"
 #include "object.h"
@@ -79,6 +80,8 @@ append_cache_item (AtkObject * obj, gpointer data)
   const char *name, *desc;
   dbus_uint32_t role;
 
+  g_object_ref (G_OBJECT (obj));
+
   set = atk_object_ref_state_set (obj);
   {
     AtkObject *application, *parent;
@@ -142,7 +145,8 @@ append_cache_item (AtkObject * obj, gpointer data)
     /* Marshall children */
     dbus_message_iter_open_container (&iter_struct, DBUS_TYPE_ARRAY, "(so)",
                                       &iter_sub_array);
-    if (!atk_state_set_contains_state (set, ATK_STATE_MANAGES_DESCENDANTS))
+    if (!atk_state_set_contains_state (set, ATK_STATE_MANAGES_DESCENDANTS) &&
+        !atk_state_set_contains_state (set, ATK_STATE_DEFUNCT))
       {
         gint childcount, i;
 
@@ -212,6 +216,7 @@ append_cache_item (AtkObject * obj, gpointer data)
   }
   dbus_message_iter_close_container (iter_array, &iter_struct);
   g_object_unref (set);
+  g_object_unref (obj);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -276,8 +281,7 @@ emit_cache_add (SpiCache *cache, GObject * obj)
 static DBusMessage *
 impl_GetRoot (DBusConnection * bus, DBusMessage * message, void *user_data)
 {
-  return spi_object_return_reference (message,
-                                      g_object_ref (G_OBJECT (spi_global_app_data->root)));
+  return spi_object_return_reference (message, spi_global_app_data->root);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -288,6 +292,9 @@ impl_GetItems (DBusConnection * bus, DBusMessage * message, void *user_data)
   DBusMessage *reply;
   DBusMessageIter iter, iter_array;
 
+  if (bus == spi_global_app_data->bus)
+    spi_atk_add_client (dbus_message_get_sender (message));
+
   reply = dbus_message_new_method_return (message);
 
   dbus_message_iter_init_append (reply, &iter);
