@@ -59,13 +59,11 @@ typedef struct _SpiReentrantCallClosure
 static void
 switch_main_context (GMainContext *cnx)
 {
-/* This code won't work on dbus-glib earlier than 0.9.0 because of FDO#30574 */
-  if (spi_global_app_data->app_bus_addr [0] == '\0')
-    return;
-
   GList *list;
 
+#ifndef DISABLE_P2P
   atspi_dbus_server_setup_with_g_main (spi_global_app_data->server, cnx);
+#endif
   atspi_dbus_connection_setup_with_g_main (spi_global_app_data->bus, cnx);
   for (list = spi_global_app_data->direct_connections; list; list = list->next)
     atspi_dbus_connection_setup_with_g_main (list->data, cnx);
@@ -391,6 +389,9 @@ signal_is_needed (const gchar *klass, const gchar *major, const gchar *minor)
         }
     }
 
+#if 0
+  g_print("event: %s %s %s: %d\n", data[0], data[1], data[2], ret);
+#endif
   g_free (data [2]);
   g_free (data [1]);
   g_free (data [0]);
@@ -605,15 +606,14 @@ state_event_listener (GSignalInvocationHint * signal_hint,
   guint detail1;
 
   accessible = ATK_OBJECT (g_value_get_object (&param_values[0]));
-  pname = g_strdup (g_value_get_string (&param_values[1]));
+  pname = g_value_get_string (&param_values[1]);
 
-  /* TODO - Possibly ignore a change to the 'defunct' state.
-   * This is because without reference counting defunct objects should be removed.
-   */
   detail1 = (g_value_get_boolean (&param_values[2])) ? 1 : 0;
   emit_event (accessible, ITF_EVENT_OBJECT, STATE_CHANGED, pname, detail1, 0,
               DBUS_TYPE_INT32_AS_STRING, 0, append_basic);
-  g_free (pname);
+
+  if (!g_strcmp0 (pname, "defunct"))
+    spi_register_deregister_object (spi_global_register, G_OBJECT (accessible));
   return TRUE;
 }
 
@@ -759,7 +759,7 @@ link_selected_event_listener (GSignalInvocationHint * signal_hint,
   AtkObject *accessible;
   GSignalQuery signal_query;
   const gchar *name, *minor;
-  gint detail1;
+  gint detail1 = 0;
 
   g_signal_query (signal_hint->signal_id, &signal_query);
   name = signal_query.signal_name;
@@ -793,7 +793,7 @@ text_changed_event_listener (GSignalInvocationHint * signal_hint,
   GSignalQuery signal_query;
   const gchar *name, *minor;
   gchar *selected;
-  gint detail1, detail2;
+  gint detail1 = 0, detail2 = 0;
 
   g_signal_query (signal_hint->signal_id, &signal_query);
   name = signal_query.signal_name;
@@ -832,7 +832,7 @@ text_insert_event_listener (GSignalInvocationHint * signal_hint,
   GSignalQuery signal_query;
   const gchar *name;
   gchar *minor, *text;
-  gint detail1, detail2;
+  gint detail1 = 0, detail2 = 0;
 
   accessible = ATK_OBJECT (g_value_get_object (&param_values[0]));
   /* Get signal name for 'Gtk:AtkText:text-changed' so
@@ -880,7 +880,7 @@ text_remove_event_listener (GSignalInvocationHint * signal_hint,
   GSignalQuery signal_query;
   const gchar *name;
   gchar *minor, *text;
-  gint detail1, detail2;
+  gint detail1 = 0, detail2 = 0;
 
   accessible = ATK_OBJECT (g_value_get_object (&param_values[0]));
   /* Get signal name for 'Gtk:AtkText:text-changed' so
@@ -931,7 +931,7 @@ text_selection_changed_event_listener (GSignalInvocationHint * signal_hint,
   AtkObject *accessible;
   GSignalQuery signal_query;
   const gchar *name, *minor;
-  gint detail1, detail2;
+  gint detail1 = 0, detail2 = 0;
 
   g_signal_query (signal_hint->signal_id, &signal_query);
   name = signal_query.signal_name;
@@ -969,7 +969,7 @@ children_changed_event_listener (GSignalInvocationHint * signal_hint,
 {
   GSignalQuery signal_query;
   const gchar *name, *minor;
-  gint detail1, detail2 = 0;
+  gint detail1 = 0, detail2 = 0;
 
   AtkObject *accessible, *ao=NULL;
   gpointer child;
@@ -1095,6 +1095,12 @@ spi_atk_register_event_listeners (void)
   g_object_unref (G_OBJECT (bo));
   g_object_unref (ao);
 
+  if (listener_ids)
+  {
+    g_warning ("atk_bridge: spi_atk-register_event_listeners called multiple times");
+    return;
+  }
+
   /* Register for focus event notifications, and register app with central registry  */
   listener_ids = g_array_sized_new (FALSE, TRUE, sizeof (guint), 16);
 
@@ -1148,10 +1154,7 @@ spi_atk_register_event_listeners (void)
                        "Gtk:AtkTable:column-reordered");
   add_signal_listener (generic_event_listener, "Gtk:AtkTable:column-deleted");
   add_signal_listener (generic_event_listener, "Gtk:AtkTable:model-changed");
-
-  /* Children signal listeners */
-  atk_add_global_event_listener (children_changed_event_listener,
-                                 "Gtk:AtkObject:children-changed");
+  add_signal_listener (children_changed_event_listener, "Gtk:AtkObject:children-changed");
 
 #if 0
   g_signal_connect (G_OBJECT (spi_global_app_data->root),
@@ -1186,15 +1189,22 @@ spi_atk_deregister_event_listeners (void)
   listener_ids = NULL;
 
   if (atk_bridge_focus_tracker_id)
+  {
     atk_remove_focus_tracker (atk_bridge_focus_tracker_id);
+    atk_bridge_focus_tracker_id = 0;
+  }
 
   for (i = 0; ids && i < ids->len; i++)
     {
       atk_remove_global_event_listener (g_array_index (ids, guint, i));
     }
+  g_array_free (ids, TRUE);
 
   if (atk_bridge_key_event_listener_id)
+  {
     atk_remove_key_event_listener (atk_bridge_key_event_listener_id);
+    atk_bridge_key_event_listener_id = 0;
+  }
 }
 
 /*---------------------------------------------------------------------------*/
